local effects = { } -- {start, duration, sourceGUID, destGUID, cooldownSpellId, spellId} indexed by sourceGUID then cooldownSpellId
local affected = { } -- same but indexed by destGUID then cooldownSpellId

local map = { }

do
	local list = PABR_GetAllSpells()

	for _, spellId in ipairs(list) do
		local info = PABR_GetSpellInfo(spellId)
		local effects = info.effect

		if type(effects) == "number" then
			map[effects] = spellId

		elseif type(effects) == "table" then
			for _, effect in ipairs(effects) do
				map[effect] = spellId
			end
		end
	end
end

-- for debug only (/dump x)
--[[
_EFFECTS1 = effects
_EFFECTS2 = affected
]]

-- API

function PABR_GetEffectInfo(guid, spellId) -- return start, duration
	local info = effects[guid]
	if info then
		info = info[spellId]
		if info then
			return info[1], info[2]
		end
	end
	return 0, 0
end

function PABR_WipeEffects()
	wipe(effects)
	wipe(affected)
end

-- Functions

local function deleteEffect(effect)
	local cooldownSpellId = effect[5]
	local list = effects[effect[3]]
	if list and list[cooldownSpellId] then
		PABR_DeleteTable(list[cooldownSpellId])
		list[cooldownSpellId] = nil
	end
	list = affected[effect[4]]
	if list then
		list[cooldownSpellId] = nil
	end
	PABR_SetDirty()
end

local function searchEffectAll(list, unit, filter, spellId)
	local found = 0
	for i=1, 40 do
		local _, _, _, _, _, duration, expiration, _, _, _, spell = UnitAura(unit, i, filter)
		if not duration then
			break
		elseif spell == spellId then
			local entry = PABR_CreateTable()
			tinsert(entry, duration)
			tinsert(entry, expiration)
			tinsert(list, entry)
			found = found + 1
		end
	end
	return found
end

function PABR_SearchEffectAll(list, unit, spellId)
	if searchEffectAll(list, unit, "HELPFUL", spellId) == 0 then
		searchEffectAll(list, unit, "HARMFUL", spellId)
	end
end

local function scanDuration(unit, effect)
	local bestStart = 0
	local bestDuration = 0
	local delta = math.huge
	local list = PABR_CreateTable()

	PABR_SearchEffectAll(list, unit, effect[6])

	for i=#list, 1, -1 do
		local start = list[i][2] - list[i][1]
		local candidate_delta = abs(start - effect[1])

		if candidate_delta < delta then
			bestStart = start
			bestDuration = list[i][1]
			delta = candidate_delta
		end

		PABR_DeleteTable(list[i])
	end

	PABR_DeleteTable(list)

	if bestDuration > 0 then
		effect[1] = bestStart
		effect[2] = bestDuration
		PABR_SetDirty()
		return 1
	end
	return -1
end

local function scanDurationUnknown(destGUID, effect)
	local unit = PABR_FindUnit(destGUID)
	return (unit and scanDuration(unit, effect)) or 0
end

-- Handlers

function PABR_UpdateEffect(unit)
	local deleteList = PABR_CreateTable()
	local guid = UnitGUID(unit)
	if guid and PABR_IsEngineEnabled() then
		local list = affected[guid]
		if list then
			for _, effect in pairs(list) do
				if scanDuration(unit, effect) < 0 then
					tinsert(deleteList, effect)
				end
			end
		end
	end
	for i=1, #deleteList do
		deleteEffect(deleteList[i])
	end
	PABR_DeleteTable(deleteList)
end

function PABR_OnEffectGain(timestamp, sourceGUID, destGUID, spellId)
	local cooldownSpellId = map[spellId]
	if cooldownSpellId then
		local effect = PABR_CreateTable()
		effect[1] = PABR_GetTime(timestamp)
		effect[2] = 0		-- unknown duration to figure out
		effect[3] = sourceGUID
		effect[4] = destGUID
		effect[5] = cooldownSpellId
		effect[6] = spellId

		if scanDurationUnknown(destGUID, effect) < 0 then
			PABR_DeleteTable(effect)
		else
			local list = effects[sourceGUID]
			if not list then
				list = { }
				effects[sourceGUID] = list
			end
			list[cooldownSpellId] = effect

			list = affected[destGUID]
			if not list then
				list = { }
				affected[destGUID] = list
			end
			list[cooldownSpellId] = effect

			PABR_SetDirty()
		end
	end
end

function PABR_OnEffectFade(timestamp, sourceGUID, destGUID, spellId)
	local cooldownSpellId = map[spellId]
	if cooldownSpellId then
		local list = effects[sourceGUID]
		if list and list[cooldownSpellId] then
			deleteEffect(list[cooldownSpellId])
		end
	end
end