local scales = {
	["ALTERACVALLEY"] = 4328,
	["WARSONGGULCH"] = 1147,
	["STRANDOFTHEANCIENTS"] = 2489,
	["TEMPLEOFKOTMOGU"] = 842,
	["GILNEASBATTLEGROUND2"] = 1304,
	["ARATHIBASIN"] = 1759,
	["TWINPEAKS"] = 1215,
	["ISLEOFCONQUEST"] = 2654,
	["STVDIAMONDMINEBG"] = 922,
	["GOLDRUSH"] = 1086,
	["NETHERSTORMARENA"] = 2278,
	
	["UNKNOWN"] = 1820, -- In case a new bg is added and we don't maintain the mod.
}

local activity = { }

local NEXT_TICK = 0

local function computeThreshold()
	local distance = PABR_Config["ActivityDistance"]
	return distance / max(distance, scales[ strupper(GetMapInfo() or "") ] or scales["UNKNOWN"])
end

local function computeDistance(unit1, unit2)
	local x1, y1 = GetPlayerMapPosition(unit1)
	if (x1 + y1) > 0 then
		local x2, y2 = GetPlayerMapPosition(unit2)
		if (x2 + y2) > 0 then
			return ((x2 - x1)^2 + ((y2 - y1)/1.5)^2)^0.5
		end
	end
	return math.huge
end

-- API

function PABR_WipeFilter()
	wipe(activity)
end

function PABR_IsFilterActive()
	local _, instanceType = GetInstanceInfo()
	return PABR_Config["EnableActivityFilter"] and instanceType == "pvp"
end

function PABR_IsFiltered(guid)
	return GetTime() - (activity[guid] or -math.huge) > PABR_Config["ActivityTimeout"]
end

-- Handlers

function PABR_CheckFilter()
	if GetTime() > NEXT_TICK and PABR_IsEngineEnabled() then
		NEXT_TICK = GetTime() + 3
		
		if not PABR_Config["EnableTest"] then
			local threshold = computeThreshold()
			
			for i=1, 2 do
				for j=1, PABR_GetTeamSize(i) do
					local guid = PABR_GetTeamMember(i, j)
					local unit, observerUnit = PABR_FindUnit(guid)
					
					if unit then
						if computeDistance(unit, "player") < threshold or computeDistance(observerUnit, "player") < threshold then
							activity[guid] = GetTime()
						end
					end
				end
			end

			PABR_SetDirty()
		end
	end
end