local team = { {}, {} }
local lookup = { {}, {} }
local summon = { }

local NEXT_PET_CHECK = 0

local function insertInTeam(team, lookup, unit)
	local guid = UnitGUID(unit)
	if guid then
		local name = UnitName(unit)
		local index = lookup[guid]
		if index then
			team[index][2] = name
		else
			local index = #team + 1
			local t = PABR_CreateTable()
			t[1] = guid
			t[2] = name
			team[index] = t
			lookup[guid] = index
			PABR_SetDirty()
		end
	end
end

local function clearFromTeam(team, lookup, unit)
	local guid = UnitGUID(unit)
	if guid then
		local index = lookup[guid]
		if index then
			-- Shift indexes in the lookup table that are above the removed index
			for guid, otherIndex in pairs(lookup) do
				if otherIndex > index then
					lookup[guid] = otherIndex - 1
				end
			end

			-- Remove the entry
			lookup[guid] = nil
			PABR_DeleteTable(team[index])
			tremove(team, index)
			PABR_SetDirty()
		end
	end
end

local function insertPet(petUnit, masterUnit)
	local petGUID = UnitGUID(petUnit)
	local masterGUID = UnitGUID(masterUnit)
	if petGUID and masterGUID then
		summon[petGUID] = masterGUID
	end
end

-- API

function PABR_GetTeamSize(teamId) -- return size
	return #team[teamId]
end

function PABR_GetTeamMember(teamId, index) -- return guid, name
	return unpack(team[teamId][index])
end

function PABR_GetTeamIndex(teamId, guid) -- return index or nil
	return lookup[teamId][guid]
end

function PABR_ResolveTeam(guid) -- return teamId, index, guid, name (nil, nil, nil, "" if not found)
	for i=1, 2 do
		local index = lookup[i][guid]
		if index then
			return i, index, PABR_GetTeamMember(i, index)
		end
	end
	return nil, nil, nil, ""
end

function PABR_ResolveGUID(guid) -- return guid (does nothing if the unit is not a pet/summon)
	local current = guid
	while summon[current] do current = summon[current] end
	return current
end

function PABR_WipeEnemyTeam()
	wipe(team[2])
	wipe(lookup[2])
end

function PABR_WipeSummon()
	wipe(summon)
end

-- Handlers

function PABR_UpdatePlayerTeam()
	wipe(team[1])
	wipe(lookup[1])

	if IsInRaid() then
		for i=1, GetNumGroupMembers() do
			insertInTeam(team[1], lookup[1], "raid"..i)
		end
	else
		for i=1, GetNumGroupMembers() do
			insertInTeam(team[1], lookup[1], "party"..i)
		end
		insertInTeam(team[1], lookup[1], "player")
	end

	PABR_InspectLocalPlayer()

	PABR_EnqueueInspect()
end

function PABR_CheckEnemyUnit(unit)
	if UnitIsPlayer(unit) then
		if UnitIsEnemy("player", unit) then
			insertInTeam(team[2], lookup[2], unit)
		else
			clearFromTeam(team[2], lookup[2], unit)
		end
	end
end

function PABR_NotifySummon(sourceGUID, destGUID)
	summon[destGUID] = sourceGUID
end

function PABR_CheckPets()
	local now = GetTime()
	if now > NEXT_PET_CHECK and PABR_IsEngineEnabled() then
		NEXT_PET_CHECK = now + 1
		
		if IsInRaid() then
			for i=1, GetNumGroupMembers() do
				insertPet("raidpet"..i, "raid"..i)
			end
		else
			for i=1, GetNumGroupMembers() do
				insertPet("partypet"..i, "party"..i)
			end
		end

		for i=1, MAX_ARENA_ENEMIES do
			insertPet("arenapet"..i, "arena"..i)
		end
	end
end

function PABR_UpdateUnitName(unit)
	local guid = UnitGUID(unit)
	if guid then
		local teamId, index = PABR_ResolveTeam(guid)
		if teamId then
			team[teamId][index][2] = UnitName(unit)
		end
	end
end