local POPUP_PROFILE = ""

local function showPopup(popupType, targetProfile)
	POPUP_PROFILE = targetProfile
	StaticPopup_Show(popupType)
end

local function updateButton(dropDown)
	local selectedProfile = UIDropDownMenu_GetSelectedValue(dropDown)
	if PABR_ActiveProfile ~= selectedProfile then
		PABR_ProfileConfigPanel_ActivateButton:Enable()
	else
		PABR_ProfileConfigPanel_ActivateButton:Disable()
	end
	if PABR_DeleteProfile(selectedProfile) then
		PABR_ProfileConfigPanel_DeleteButton:Enable()
	else
		PABR_ProfileConfigPanel_DeleteButton:Disable()
	end
end

StaticPopupDialogs["PABR_CLONE_PROFILE"] = {
	text = "Enter the name to give to the cloned profile:",
	button1 = OKAY,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	hasEditBox = 1,
	maxLetters = 32,
	cancels = "PABR_DELETE_PROFILE",
	hideOnEscape = 1,

	OnAccept = function(self)
		PABR_CloneProfile(POPUP_PROFILE, self.editBox:GetText(), true)
		updateButton(PABR_ProfileConfigPanel_ProfileDropDown)
	end,
	OnShow = function(self)
		self.editBox:SetFocus()
		self.editBox:SetText("")
		self.button1:Disable()
	end,
	EditBoxOnTextChanged = function(self)
		local button = self:GetParent().button1
		if PABR_CloneProfile(POPUP_PROFILE, self:GetText()) then
			button:Enable()
		else
			button:Disable()
		end
	end,
	EditBoxOnEnterPressed = function(self)
		PABR_CloneProfile(POPUP_PROFILE, self:GetText(), true)
		updateButton(PABR_ProfileConfigPanel_ProfileDropDown)
		self:GetParent():Hide()
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide()
	end,
}

StaticPopupDialogs["PABR_DELETE_PROFILE"] = {
	text = "Are you sure you want to delete this profile?",
	button1 = OKAY,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	cancels = "PABR_CLONE_PROFILE",
	hideOnEscape = 1,

	OnAccept = function(self)
		PABR_Config_SaveAll()
		PABR_DeleteProfile(POPUP_PROFILE, true)
		PABR_CheckEnable()
		PABR_Config_RefreshAll()
	end,
}

StaticPopupDialogs["PABR_RESET"] = {
	text = "This will completely reset PABR configuration to initial settings, |cffff8000ALL PROFILES WILL BE WIPED|r. Are you sure you want to continue?",
	button1 = OKAY,
	button2 = CANCEL,
	timeout = 0,
	whileDead = 1,
	exclusive = 1,
	showAlert = 1,
	hideOnEscape = 1,

	OnAccept = function(self)
		PABR_WipeProfiles()
		PABR_CheckEnable()
		PABR_Config_RefreshAll()
	end,
}

-- Functions

function PABR_ProfileConfigPanel_OnLoad(self)
	self.name = "Profiles"
	self.parent = "PAB Reloaded"
	self.okay = function() PABR_CheckEnable() end -- Check needs to be done after clicking Okay but preferably after all PABR settings have been processed, so Profiles is a good place to put it.
	self.cancel = function() end
	self.default = function() end
	InterfaceOptions_AddCategory(self)

	PABR_ProfileConfigPanel_ActivateButton:SetText("Activate")
	PABR_ProfileConfigPanel_CloneButton:SetText("Clone")
	PABR_ProfileConfigPanel_DeleteButton:SetText("Delete")
	PABR_ProfileConfigPanel_ResetButton:SetText("|cffff8000Reset PABR|r")
end

function PABR_ProfileConfigPanel_Refresh(self)
	_G[self:GetName().."_CurrentProfile"]:SetText(("Active profile for |cffffff00%s|r: |cff00ff00%s|r"):format(UnitName("player"), PABR_ActiveProfile))

	local dropDown = _G[self:GetName().."_ProfileDropDown"]
	UIDropDownMenu_SetWidth(dropDown, 200)
	UIDropDownMenu_Initialize(dropDown, PABR_ProfileConfigPanel_ProfileDropDown_Initialize)
	UIDropDownMenu_SetSelectedValue(dropDown, PABR_ActiveProfile)
	updateButton(dropDown)
end

function PABR_ProfileConfigPanel_ActivateButton_OnClick(self)
	PABR_Config_SaveAll()

	PABR_SetActiveProfile(UIDropDownMenu_GetSelectedValue(_G[self:GetParent():GetName().."_ProfileDropDown"]))

	-- Reevaluate activation of the mod
	PABR_CheckEnable()

	PABR_Config_RefreshAll()
end

function PABR_ProfileConfigPanel_CloneButton_OnClick(self)
	showPopup("PABR_CLONE_PROFILE", UIDropDownMenu_GetSelectedValue(_G[self:GetParent():GetName().."_ProfileDropDown"]))
end

function PABR_ProfileConfigPanel_DeleteButton_OnClick(self)
	showPopup("PABR_DELETE_PROFILE", UIDropDownMenu_GetSelectedValue(_G[self:GetParent():GetName().."_ProfileDropDown"]))
end

function PABR_ProfileConfigPanel_ResetButton_OnClick(self)
	showPopup("PABR_RESET", "")
end

function PABR_ProfileConfigPanel_ProfileDropDown_Initialize(self)
	local selectedValue = UIDropDownMenu_GetSelectedValue(self)
	local info = UIDropDownMenu_CreateInfo()
	local names = PABR_GetProfileNames()

	for index, name in ipairs(names) do
		info.value = name
		info.text = name
		info.checked = info.value == selectedValue
		info.func = PABR_ProfileConfigPanel_ProfileDropDown_OnClick
		info.arg1 = self
		UIDropDownMenu_AddButton(info)
	end
end

function PABR_ProfileConfigPanel_ProfileDropDown_OnClick(self)
	UIDropDownMenu_SetSelectedValue(self.arg1, self.value)
	updateButton(self.arg1)
end