local checkButtons = {
	"EnableTest",
	"UILocked",

	"ShowLocalPlayer",
	"EnableInspect",
	"EnableActivityFilter"
}

local sliders = {
	{"ActivityTimeout", "%d sec", 10, 300, 5},
	{"ActivityDistance", "%d yds", 30, 500, 10},
}

local displays = {
	{0, "Disable user interface"},
	{1, "Show my team cooldowns"},
	{2, "Show enemy team cooldowns"},
	{3, "Show both teams cooldowns"},
}

local dropDowns = {
	{"EnableArena", displays},
	{"EnableRatedBg", displays},
	{"EnableBg", displays},
	{"EnableRaid", displays},
	{"EnableParty", displays},
	{"EnableSolo", displays},
}

-- Services

local function DropDown_OnClick(self)
	UIDropDownMenu_SetSelectedValue(self.arg1, self.value)
end

local function DropDown_Initialize(self)
	local dropDown = UIDropDownMenu_GetCurrentDropDown()
	local selectedValue = UIDropDownMenu_GetSelectedValue(self)
	local info = UIDropDownMenu_CreateInfo()

	for index, option in ipairs(dropDown.generator) do
		info.value = option[1]
		info.text = option[2]
		info.checked = info.value == selectedValue
		info.func = DropDown_OnClick
		info.arg1 = self
		UIDropDownMenu_AddButton(info)
	end
end

function PABR_Config_InitWidget(self, type, list)
	if type == "button" then
		-- Nothing to do
	elseif type == "slider" then
		for index, option in ipairs(list) do
			local slider = _G[self:GetName().."_"..option[1].."Slider"]
			slider.format = option[2]
			slider:SetMinMaxValuesEx(option[3], option[4])
			slider:SetValueStep(option[5])
		end
	elseif type == "dropdown" then
		-- Nothing to do
	end
end

function PABR_Config_LoadWidget(self, type, list)
	if type == "button" then
		for index, option in ipairs(list) do
			_G[self:GetName().."_"..option.."CheckButton"]:SetChecked(PABR_Config[option])
		end
	elseif type == "slider" then
		for index, option in ipairs(list) do
			_G[self:GetName().."_"..option[1].."Slider"]:SetValue(PABR_Config[option[1]])
		end
	elseif type == "dropdown" then
		for index, option in ipairs(list) do
			local dropDown = _G[self:GetName().."_"..option[1].."DropDown"]
			dropDown.generator = option[2]
			UIDropDownMenu_SetWidth(dropDown, 200)
			UIDropDownMenu_Initialize(dropDown, DropDown_Initialize)
			UIDropDownMenu_SetSelectedValue(dropDown, PABR_Config[option[1]])
		end
	end
end

function PABR_Config_SaveWidget(self, type, list)
	if type == "button" then
		for index, option in ipairs(list) do
			PABR_Config[option] = not not _G[self:GetName().."_"..option.."CheckButton"]:GetChecked()
		end
	elseif type == "slider" then
		for index, option in ipairs(list) do
			PABR_Config[option[1]] = _G[self:GetName().."_"..option[1].."Slider"]:GetValue()
		end
	elseif type == "dropdown" then
		for index, option in ipairs(list) do
			PABR_Config[option[1]] = UIDropDownMenu_GetSelectedValue(_G[self:GetName().."_"..option[1].."DropDown"])
		end
	end
end

function PABR_Config_ResetWidget(self, type, list)
	if type == "button" then
		for index, option in ipairs(list) do
			PABR_Config[option] = PABR_DefaultConfig[option]
		end
	elseif type == "slider" or type == "dropdown" then
		for index, option in ipairs(list) do
			PABR_Config[option[1]] = PABR_DefaultConfig[option[1]]
		end
	end
	PABR_Config_LoadWidget(self, type, list)
end

-- Functions

function PABR_Config_SaveAll()
	PABR_ConfigPanel_Okay(PABR_ConfigPanel)
	PABR_InterfaceConfigPanel_Okay(PABR_InterfaceConfigPanel)
	PABR_AlertsConfigPanel_Okay(PABR_AlertsConfigPanel)
	PABR_SpellConfigPanel_Okay(PABR_SpellConfigPanel)
end

function PABR_Config_RefreshAll()
	PABR_ConfigPanel_Refresh(PABR_ConfigPanel)
	PABR_InterfaceConfigPanel_Refresh(PABR_InterfaceConfigPanel)
	PABR_AlertsConfigPanel_Refresh(PABR_AlertsConfigPanel)
	PABR_SpellConfigPanel_Refresh(PABR_SpellConfigPanel)
	PABR_ProfileConfigPanel_Refresh(PABR_ProfileConfigPanel)
end

function PABR_ConfigPanel_OnLoad(self)
	self.name = "PAB Reloaded"
	self.okay = PABR_ConfigPanel_Okay
	self.cancel = PABR_ConfigPanel_Refresh
	self.default = PABR_ConfigPanel_Default
	InterfaceOptions_AddCategory(self)
	
	PABR_Config_InitWidget(self, "button", checkButtons)
	PABR_Config_InitWidget(self, "slider", sliders)
	PABR_Config_InitWidget(self, "dropdown", dropDowns)
	
	PABR_ConfigPanel_EnableTestCheckButton:SetText("|cff00ff00Test|r user interface")
	PABR_ConfigPanel_UILockedCheckButton:SetText("|cff00ff00Lock|r user interface")

	PABR_ConfigPanel_ShowLocalPlayerCheckButton:SetText("Show your own character cooldowns")
	PABR_ConfigPanel_EnableInspectCheckButton:SetText("Enable inspections (better simulations for your team)")
	PABR_ConfigPanel_EnableActivityFilterCheckButton:SetText("Enable filter (only for battlefields - hide cooldowns of players far away from you)")
	PABR_ConfigPanel_ActivityTimeoutSlider:SetText("Filter delay")
	PABR_ConfigPanel_ActivityDistanceSlider:SetText("Filter distance")

	PABR_ConfigPanel_EnableArenaText:SetText("When I am in an arena...")
	PABR_ConfigPanel_EnableRatedBgText:SetText("When I am in a rated battlefield...")
	PABR_ConfigPanel_EnableBgText:SetText("When I am in a normal battlefield...")
	PABR_ConfigPanel_EnableRaidText:SetText("When I am in a raid...")
	PABR_ConfigPanel_EnablePartyText:SetText("When I am in a party...")
	PABR_ConfigPanel_EnableSoloText:SetText("When I am alone...")
end

function PABR_ConfigPanel_Okay(self)
	PABR_Config_SaveWidget(self, "button", checkButtons)
	PABR_Config_SaveWidget(self, "slider", sliders)
	PABR_Config_SaveWidget(self, "dropdown", dropDowns)
end

function PABR_ConfigPanel_Default(self)
	PABR_Config_ResetWidget(self, "button", checkButtons)
	PABR_Config_ResetWidget(self, "slider", sliders)
	PABR_Config_ResetWidget(self, "dropdown", dropDowns)
	
	PABR_CheckEnable()
end

function PABR_ConfigPanel_Refresh(self)
	PABR_Config_LoadWidget(self, "button", checkButtons)
	PABR_Config_LoadWidget(self, "slider", sliders)
	PABR_Config_LoadWidget(self, "dropdown", dropDowns)
end